'''
Created on Feb 5, 2013

@author: Trung Dong Huynh
'''
from HTMLParser import HTMLParser
from collections import defaultdict
import csv

class WBSParser(HTMLParser):
    def __init__(self):
        HTMLParser.__init__(self)
        self.data = defaultdict(dict)
        self.responses = None
        self.qcounter = 0
        self.rcounter = 0
        self.headers = None
        self.current_response = None
        self.parsing = False
        self.parsing_header = False
        self.parsing_response = False
        self.parsing_data = False
        self.has_break = False
        
    def handle_data(self, data):
        if self.parsing_data:
            if data == '\n' or data == '\r\n':
                pass
            else:
                data = data.lstrip('\r\n')
                if self.parsing_header:
                    self.headers.append(data)
                elif self.parsing_response:
                    if not self.has_break:
                        self.current_response.append(data)
                    else:
                        self.current_response[-1] += '\n'
                        self.current_response[-1] += data
            self.has_break = False
    
    def handle_starttag(self, tag, attrs):
        if self.parsing:
            if tag == 'td' or tag == 'th':
                self.parsing_data = True;
            elif tag == 'br':
                self.has_break = True
            elif tag == 'tr' and not self.parsing_header:
                self.parsing_response = True
                self.rcounter += 1 
            elif tag == 'thead':
                self.parsing_header = True
        elif tag == 'table':
            # Check if this is the result table
            if ('class', 'results') in attrs:
                self.parsing = True
                # Resetting everything
                self.qcounter += 1
                self.rcounter = 0
                self.headers = []
                self.responses = []
                self.current_response = []
        
    def handle_endtag(self, tag):
        if self.parsing:
            if tag == 'td' or tag == 'th':
                self.parsing_data = False;
            elif tag == 'tr' and self.parsing_response:
                self.parsing_response = False
                # Record the current_response
                self.responses.append(self.current_response)
                self.current_response = []
            elif tag == 'thead':
                self.parsing_header = False
            elif tag == 'table':
                self.parsing = False
                self.data[self.qcounter]['headers'] = self.headers
                self.data[self.qcounter]['responses'] = self.responses
                print self.headers
                print self.responses 

def parse_wbs_results(filename, output):
    with open(filename) as f:
        parser = WBSParser()
        parser.feed(f.read())
        parser.close()
    write_csv(parser.data, output)

def write_csv(data, filename):
    with open(filename, 'wb') as csvfile:
        csvwriter = csv.writer(csvfile)
        for qid, qdata in data.iteritems():
            csvwriter.writerow(('Question %d:' % qid,))
            csvwriter.writerow(qdata['headers'])
            csvwriter.writerows(qdata['responses'])
            csvwriter.writerow([])
    
if __name__ == '__main__':
    parse_wbs_results('implementations.htm', 'implementations.csv')
    parse_wbs_results('vocabulary-extensions.htm', 'vocabulary-extensions.csv')
    parse_wbs_results('vocabulary-usages.htm', 'vocabulary-usages.csv')